/* 
 * Copyright (C) 2004 Shingo W. Kagami.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: gkbinclock.c,v 1.2 2004/08/27 19:53:57 swk Exp $
 */

#include <gkrellm/gkrellm.h>

#define	CONFIG_NAME	"gkbinclock"
#define	STYLE_NAME	"gkbinclock"
#define MAX_LED_NO 16

static gint style_id;
static Monitor *monitor;
static Panel *panel;
static Decal *led[MAX_LED_NO];

static gint prev_hour, prev_min, prev_sec;
static gint prev_bitpattern;
static gint led_no;

static gint
panel_expose_event(GtkWidget *widget, GdkEventExpose *ev)
{
    gdk_draw_pixmap(widget->window,
		    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		    panel->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
		    ev->area.width, ev->area.height);
    return FALSE;
}

static void
set_led()
{
    struct tm *tm;
    gint i;
    gint hour, min, sec;
    gint bitpattern, bitdiff;

    tm = gkrellm_get_current_time();
    hour = tm->tm_hour % 12;
    min = tm->tm_min;
    sec = tm->tm_sec;

    if (sec == prev_sec) {
	return; 
    }

    /* pure binary format */
    bitpattern = ((hour & 0x0f) << 12) | ((min & 0x3f) << 6) | (sec & 0x3f);


    bitdiff = bitpattern ^ prev_bitpattern;
    for (i = 0; i < led_no; i++) {
	if (bitdiff & (1 << i)) {
	    gkrellm_draw_decal_pixmap(panel, led[i],
				      (bitpattern & (1 << i))?
				      D_MISC_LED1: D_MISC_LED0);
	}
    }
    prev_hour = hour;
    prev_min = min;
    prev_sec = sec;
    prev_bitpattern = bitpattern;
}

static void
update_plugin()
{
    set_led();
    gkrellm_draw_panel_layers(panel);
}

static void
locate_led_binary(gint *xs, gint *ys, gint *ptr_led_no)
{
    gint i;
    gint x, y;

    /* = indices to led[] =
     *
     *        15 14 13 12    hour
     *  11 10  9  8  7  6    min
     *   5  4  3  2  1  0    sec
     *  MSB <------->  LSB
     *
     */
    
    *ptr_led_no = 16;
    
    /* hour */
    x = 20; y = 3; 
    for (i = 15; i >= 12; i--) {
	xs[i] = x;
	ys[i] = y;	
	x += 8;
    }
    /* min */
    x = 4; y = 3 + 6; 
    for (i = 11; i >= 6; i--) {
	xs[i] = x;
	ys[i] = y;	
	x += 8;
    }
    /* sec */
    x = 4; y = 3 + 6 * 2; 
    for (i = 5; i >= 0; i--) {
	xs[i] = x;
	ys[i] = y;	
	x += 8;
    }
}

static void
create_plugin(GtkWidget *vbox, gint first_create)
{
    Style *style;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    gint xs[MAX_LED_NO], ys[MAX_LED_NO];
    gint i;

    if (first_create) {
	panel = gkrellm_panel_new0();
    }

    style = gkrellm_meter_style(style_id);
    pixmap = gkrellm_decal_misc_pixmap();
    mask = gkrellm_decal_misc_mask();

    /* pure binary format */
    locate_led_binary(xs, ys, &led_no);

    for (i = 0; i < led_no; i++) {
	led[i] = gkrellm_create_decal_pixmap(panel, pixmap, mask,
					     N_MISC_DECALS, NULL,
					     xs[i], ys[i]);
    }

    gkrellm_panel_configure(panel, NULL, style);
    gkrellm_panel_create(vbox, monitor, panel);

    for (i = 0; i < led_no; i++) {
	gkrellm_draw_decal_pixmap(panel, led[i], D_MISC_LED0);
    }
    prev_hour = prev_min = prev_sec = 0;
    prev_bitpattern = 0;

    if (first_create) {
	gtk_signal_connect(GTK_OBJECT (panel->drawing_area), "expose_event",
			   (GtkSignalFunc) panel_expose_event, NULL);
    }
}


static Monitor plugin_mon =
{
    CONFIG_NAME,   /* Name, for config tab. */
    0,		   /* Id, 0 if a plugin */
    create_plugin, /* The create function */
    update_plugin, /* The update function */
    NULL,	   /* The config tab create function */
    NULL,	   /* Apply the config function */
    NULL,	   /* Save user config */
    NULL,	   /* Load user config */
    NULL,	   /* config keyword */
    NULL,	   /* Undefined 2 */
    NULL,	   /* Undefined 1 */
    NULL,	   /* private */
    MON_CPU,	   /* Insert plugin before this monitor */
    NULL,   	   /* Handle if a plugin, filled in by GKrellM */
    NULL	   /* path if a plugin, filled in by GKrellM */
};


Monitor *
init_plugin()
{
    style_id = gkrellm_add_meter_style(&plugin_mon, STYLE_NAME);
    monitor = &plugin_mon;
    return &plugin_mon;
}
